//-------------------------------------------------------------------------------------------------
// <copyright file="elevatation.h" company="Microsoft">
//    Copyright (c) Microsoft Corporation.  All rights reserved.
// </copyright>
// 
// <summary>
//    Burn elevation process handler.
// </summary>
//-------------------------------------------------------------------------------------------------

#pragma once


#ifdef __cplusplus
extern "C" {
#endif

typedef enum _BURN_ELEVATION_MESSAGE_TYPE
{
    BURN_ELEVATION_MESSAGE_TYPE_UNKNOWN,
    BURN_ELEVATION_MESSAGE_TYPE_CACHE,
    BURN_ELEVATION_MESSAGE_TYPE_EXECUTE,
    BURN_ELEVATION_MESSAGE_TYPE_ROLLBACK,

    BURN_ELEVATION_MESSAGE_TYPE_EXECUTE_PACKAGE_BEGIN,
    BURN_ELEVATION_MESSAGE_TYPE_EXECUTE_PACKAGE_ERROR,
    BURN_ELEVATION_MESSAGE_TYPE_EXECUTE_PACKAGE_PROGRESS,
    BURN_ELEVATION_MESSAGE_TYPE_EXECUTE_PACKAGE_MSI_MESSAGE,
    BURN_ELEVATION_MESSAGE_TYPE_EXECUTE_PACKAGE_COMPLETE,
    BURN_ELEVATION_MESSAGE_TYPE_RESTART_REQUIRED,
    BURN_ELEVATION_MESSAGE_TYPE_COMPLETE,

    BURN_ELEVATION_MESSAGE_TYPE_CHILD_UNINITIALIZE,
} BURN_ELEVATION_MESSAGE_TYPE;

typedef struct _BURN_ELEVATION_MESSAGE
{
    DWORD dwMessage;
    DWORD cbData;

    BOOL fAllocatedData;
    LPVOID pvData;
} BURN_ELEVATION_MESSAGE;


// Common functions.
void ElevationMessageUninitialize(
    __in BURN_ELEVATION_MESSAGE* pMsg
    );

// Parent (user mode) side functions.
HRESULT ElevationParentProcessConnect(
    __in HWND hwndParent,
    __in IInstallationAbstractionLayer* pInstallationAbstractionLayer,
    __out HANDLE* phElevationProcess,
    __out HANDLE* phElevationPipe
    );

HRESULT ElevationParentPackageExecute(
    __in HANDLE hElevationProcess,
    __in HANDLE hElevationPipe,
    __in IBurnModel* pModel,
    __in IBurnView* pView,
    __in IBurnPackage* pPackage
    );

HRESULT ElevationParentPackageRollback(
    __in HANDLE hElevationProcess,
    __in HANDLE hElevationPipe,
    __in IBurnModel* pModel,
    __in IBurnView* pView,
    __in IBurnPackage* pPackage
    );

// Child (elevated mode) side functions.
HRESULT ElevationChildGetMessage(
    __in HANDLE hPipe,
    __in BURN_ELEVATION_MESSAGE* pMsg
    );

HRESULT ElevationChildPostMessage(
    __in HANDLE hPipe,
    __in DWORD dwMessage,
    __in_bcount_opt(cbData) LPVOID pvData,
    __in DWORD cbData
    );

HRESULT ElevationChildSendMessage(
    __in HANDLE hPipe,
    __in DWORD dwMessageType,
    __in_bcount_opt(cbData) LPVOID pvData,
    __in DWORD cbData,
    __out int* pnResult
    );

#ifdef __cplusplus
}
#endif
