//-------------------------------------------------------------------------------------------------
// <copyright file="IBurnExecuteEngine.h" company="Microsoft">
//    Copyright (c) Microsoft Corporation.  All rights reserved.
//    
//    The use and distribution terms for this software are covered by the
//    Common Public License 1.0 (http://opensource.org/licenses/cpl.php)
//    which can be found in the file CPL.TXT at the root of this distribution.
//    By using this software in any fashion, you are agreeing to be bound by
//    the terms of this license.
//    
//    You must not remove this notice, or any other, from this software.
// </copyright>
//-------------------------------------------------------------------------------------------------

#pragma once

interface IBurnPackage; // forward declare.
interface IBurnProperties;
interface IBurnView;
interface IBurnModel;

DECLARE_INTERFACE_IID_(IBurnExecuteEngine, IUnknown, "e1e09b87-3fca-11dd-8291-001d09081dd9")
{
    STDMETHOD(Detect)(
        __in IBurnPackage* pPackage,
        __in IBurnProperties* pProperties,
        __out PACKAGE_STATE* pInstalledState,
        __out DWORD64* pqwInstalledVersion
        ) PURE;

    STDMETHOD(Plan)(
        __in IBurnPackage* pPackage,
        __in IBurnProperties* pProperties,
        __in REQUEST_STATE requested,
        __out ACTION_STATE* pExecute,
        __out ACTION_STATE* pRollback
        ) PURE;

    STDMETHOD(BeginTransaction)(
        __out HANDLE* phChangeOfOwnerEvent
        ) PURE;

    STDMETHOD(JoinTransaction)(
        __out HANDLE* phChangeOfOwnerEvent
        ) PURE;

    STDMETHOD(EndTransaction)(
        __in INSTALL_TRANSACTION transactionState
        ) PURE;

    STDMETHOD(Execute)(
        __in IBurnPackage* pPackage,
        __in IBurnView* pView
        ) PURE;

    STDMETHOD(Rollback)(
        __in IBurnPackage* pPackage,
        __in IBurnView* pView
        ) PURE;
};


HRESULT BurnCreateMsiEngine(
    __in IInstallationAbstractionLayer* pInstallationAbstractionLayer,
    __out IBurnExecuteEngine** ppEngine
    );

HRESULT BurnCreateExeEngine(
    __in IInstallationAbstractionLayer* pInstallationAbstractionLayer,
    __out IBurnExecuteEngine** ppEngine
    );
