//-------------------------------------------------------------------------------------------------
// <copyright file="IBurnModel.h" company="Microsoft">
//    Copyright (c) Microsoft Corporation.  All rights reserved.
//    
//    The use and distribution terms for this software are covered by the
//    Common Public License 1.0 (http://opensource.org/licenses/cpl.php)
//    which can be found in the file CPL.TXT at the root of this distribution.
//    By using this software in any fashion, you are agreeing to be bound by
//    the terms of this license.
//    
//    You must not remove this notice, or any other, from this software.
// </copyright>
//-------------------------------------------------------------------------------------------------

#pragma once

enum BURN_MODE
{
    BURN_MODE_NORMAL,
    BURN_MODE_ELEVATED,
    BURN_MODE_EMBEDDED,
};


DECLARE_INTERFACE_IID_(IBurnModel, IUnknown, "e1e09b82-3fca-11dd-8291-001d09081dd9")
{
    STDMETHOD(GetCommand)(
        __out BURN_COMMAND* pCommand
        ) PURE;

    STDMETHOD(GetCommandLineParameters)(
        __out_z LPWSTR* psczCommandLine
        ) PURE;

    STDMETHOD(GetElevatedInfo)(
        __out_z LPWSTR* psczPipeName,
        __out_z LPWSTR* psczToken
        ) PURE;

    STDMETHOD(GetLogPath)(
        __out_z LPWSTR* psczLogPath,
        __out_opt BURN_LOGGING_LEVEL* pLogLevel
        ) PURE;

    STDMETHOD_(BURN_MODE, Mode)() PURE;

    STDMETHOD(GetPackage)(
        __in DWORD dwIndex,
        __out IBurnPackage** ppPackage
        ) PURE;

    STDMETHOD(GetPackageIndex)(
        __in_z LPCWSTR wzId,
        __out DWORD* pdwIndex
        ) PURE;

    STDMETHOD(GetProperties)(
        __out IBurnProperties** ppProperties
        ) PURE;

    STDMETHOD_(DWORD, PackageCount)() PURE;

    STDMETHOD_(DWORD, UXPayloadCount)() PURE;

    STDMETHOD(GetUXPayload)(
        __in DWORD dwIndex,
        __out IBurnPayload** pPayload
        ) PURE;

    STDMETHOD(Load)(
        __in_z LPCWSTR sczBundleName
        ) PURE;

    STDMETHOD(InitializeLog)(
        __in BURN_LOGGING_LEVEL logLevel,
        __in BURN_LOGGING_MODE logMode,
        __in_z_opt LPCWSTR wzLogPath
        ) PURE;
};


DECLARE_INTERFACE_IID_(IBurnModelTestable, IUnknown, "f1e09b82-3fca-11dd-8291-001d09081dd9")
{
    STDMETHOD(AddPackageForTestingPurposes)(
        __in IBurnPackage* pPackage
        ) PURE;
};


HRESULT BurnCreateModel(
    __in_z LPCWSTR wzCommandLine,
    __out IBurnModel** ppModel
    );
