//-------------------------------------------------------------------------------------------------
// <copyright file="IBurnPackage.h" company="Microsoft">
//    Copyright (c) Microsoft Corporation.  All rights reserved.
//    
//    The use and distribution terms for this software are covered by the
//    Common Public License 1.0 (http://opensource.org/licenses/cpl.php)
//    which can be found in the file CPL.TXT at the root of this distribution.
//    By using this software in any fashion, you are agreeing to be bound by
//    the terms of this license.
//    
//    You must not remove this notice, or any other, from this software.
// </copyright>
//-------------------------------------------------------------------------------------------------

#pragma once

interface IBurnPayload; // forward declare.

enum BURN_LOGGING_LEVEL
{
    BURN_LOGGING_LEVEL_UNKNOWN,
    BURN_LOGGING_LEVEL_NORMAL,
    BURN_LOGGING_LEVEL_VERBOSE,
    BURN_LOGGING_LEVEL_DEBUG,
};

enum BURN_LOGGING_MODE
{
    BURN_LOGGING_MODE_UNKNOWN,
    BURN_LOGGING_MODE_OVERWRITE,
    BURN_LOGGING_MODE_APPEND,
    BURN_LOGGING_MODE_UNIQUE,
};

// IBurnPackage isn't a real QI'able interface.
interface IBurnPackage : IUnknown
{
    STDMETHOD_(LPCWSTR, Id)() PURE;

    STDMETHOD_(int, CompareId)(
        __in_z LPCWSTR wzId
        ) PURE;

    STDMETHOD(Detected)(
        __in PACKAGE_STATE state,
        __in DWORD64 qwInstalledVersion
        ) PURE;

    STDMETHOD_(BOOL, IsDetected)() PURE;

    STDMETHOD_(BOOL, IsPerMachine)() PURE;

    STDMETHOD_(void, GetInstalledState)(
        __out_opt PACKAGE_STATE* pState,
        __out_opt DWORD64* pqwVersion
        ) PURE;

    STDMETHOD_(void, GetActionState)(
        __out_opt ACTION_STATE* pExecute,
        __out_opt ACTION_STATE* pRollback
        ) PURE;

    STDMETHOD(GetProperties)(
        __out IBurnProperties** ppProperties
        ) PURE;

    STDMETHOD(GetPayload)(
        __in DWORD dwIndex,
        __out IBurnPayload** ppPayload
        ) PURE;

    STDMETHOD_(DWORD, PayloadCount)() PURE;

    STDMETHOD_(void, GetVersion)(
        __out DWORD64* pqwVersion
        ) PURE;

    STDMETHOD_(void, SetActionState)(
        __in ACTION_STATE execute,
        __in ACTION_STATE rollback
        ) PURE;

    STDMETHOD(InitializeLog)(
        __in BURN_LOGGING_LEVEL logLevel,
        __in BURN_LOGGING_MODE logMode,
        __in LPCWSTR wzChainLogPath,
        __in_opt LPCWSTR wzLogPath
        ) PURE;

    STDMETHOD(GetLogPath)(
        __out LPWSTR* psczLogPath,
        __out_opt BURN_LOGGING_LEVEL* pLogLevel
        ) PURE;
};


DECLARE_INTERFACE_IID_(IBurnPackageTestable, IUnknown, "f1e09b85-3fca-11dd-8291-001d09081dd9")
{
    STDMETHOD(AddPayloadForTestingPurposes)(
        __in IBurnPayload* pPayload
        ) PURE;
};
