//-------------------------------------------------------------------------------------------------
// <copyright file="IBurnView.h" company="Microsoft">
//    Copyright (c) Microsoft Corporation.  All rights reserved.
//    
//    The use and distribution terms for this software are covered by the
//    Common Public License 1.0 (http://opensource.org/licenses/cpl.php)
//    which can be found in the file CPL.TXT at the root of this distribution.
//    By using this software in any fashion, you are agreeing to be bound by
//    the terms of this license.
//    
//    You must not remove this notice, or any other, from this software.
// </copyright>
//-------------------------------------------------------------------------------------------------

#pragma once

interface IBurnController; // forward declare.

DECLARE_INTERFACE_IID_(IBurnView, IUnknown, "e1e09b83-3fca-11dd-8291-001d09081dd9")
{
    STDMETHOD(Initialize)(
        __in IBurnModel* pModel,
        __in IBurnController* pController
        ) PURE;

    STDMETHOD_(void, Uninitialize)() PURE;

    STDMETHOD(Run)() PURE;

    //
    // Prepare callbacks.
    //
    STDMETHOD_(int, OnPrepareBegin)(
        __in DWORD cPackages
        ) PURE;

    STDMETHOD_(int, OnPreparePackageBegin)(
        __in IBurnPackage* pPackage
        ) PURE;

    STDMETHOD_(void, OnPreparePackageComplete)(
        __in IBurnPackage* pPackage,
        __in HRESULT hrStatus,
        __in PACKAGE_STATE state,
        __in REQUEST_STATE requested,
        __in ACTION_STATE execute,
        __in ACTION_STATE rollback
        ) PURE;

    STDMETHOD_(void, OnPrepareComplete)(
        __in HRESULT hrStatus
        ) PURE;

    //
    // Cache callbacks.
    //
    STDMETHOD_(int, OnCacheBegin)(
        __in DWORD cPackages,
        __in DWORD cTotalPayloads,
        __in DWORD64 dw64TotalCacheSize
        ) PURE;

    STDMETHOD_(int, OnCachePackageBegin)(
        __in IBurnPackage* pPackage,
        __in DWORD cPayloads,
        __in DWORD64 dw64PackageCacheSize
        ) PURE;

    STDMETHOD_(int, OnCachePackagePayloadBegin)(
        __in_opt IBurnPackage* pPackage,
        __in IBurnPayload* pPayload,
        __in SOURCE_TYPE sourceType,
        __in_z LPCWSTR wzSourcePath
        ) PURE;

    STDMETHOD_(int, OnCacheProgress)(
        __in_opt IBurnPackage* pPackage,
        __in IBurnPayload* pPayload,
        __in DWORD64 dw64PayloadCached,
        __in DWORD64 dw64PayloadSize
        ) PURE;

    STDMETHOD_(void, OnCachePackagePayloadComplete)(
        __in_opt IBurnPackage* pPackage,
        __in IBurnPayload* pPayload,
        __in HRESULT hrExitCode
        ) PURE;

    STDMETHOD_(void, OnCachePackageComplete)(
        __in IBurnPackage* pPackage,
        __in HRESULT hrExitCode
        ) PURE;

    STDMETHOD_(void, OnCacheComplete)(
        __in HRESULT hrStatus
        ) PURE;

    //
    // Execute callbacks.
    //
    STDMETHOD_(int, OnExecuteBegin)(
        __in DWORD cExecutingPackages
        ) PURE;

    STDMETHOD_(int, OnExecutePackageBegin)(
        __in IBurnPackage* pPackage,
        __in BOOL fExecute
        ) PURE;

    STDMETHOD_(int, OnExecuteError)(
        __in IBurnPackage* pPackage,
        __in DWORD dwCode,
        __in_z LPCWSTR wzMessage,
        __in UINT uiFlags
        ) PURE;

    STDMETHOD_(int, OnExecuteProgress)(
        __in IBurnPackage* pPackage,
        __in DWORD dwPercentage
        ) PURE;

    STDMETHOD_(int, OnExecuteMsiMessage)(
        __in IBurnMsiPackage* pPackage,
        __in INSTALLMESSAGE mt,
        __in UINT uiFlags,
        __in_z LPCWSTR wzMessage,
        __in_opt MSIHANDLE hRecord
        ) PURE;

    STDMETHOD_(void, OnExecutePackageComplete)(
        __in IBurnPackage* pPackage,
        __in HRESULT hrExitCode
        ) PURE;

    STDMETHOD_(void, OnExecuteComplete)(
        __in HRESULT hrStatus
        ) PURE;

    STDMETHOD_(BOOL, OnRestartRequired)() PURE;

    STDMETHOD(GetControlText)(
        __in DWORD dwControlID,
        __out LPWSTR* psczText
        ) PURE;

    STDMETHOD(WaitIfPaused)() PURE;
};

#define HRESULT_FROM_VIEW(n) IDOK == n || IDNOACTION == n ? S_OK : IDCANCEL == n || IDABORT == n ? HRESULT_FROM_WIN32(ERROR_INSTALL_USEREXIT) : HRESULT_FROM_WIN32(ERROR_INSTALL_FAILURE)

HRESULT BurnCreateView(
    __in int nCmdShow,
    __out IBurnView** ppView
    );

HRESULT BurnCreateViewElevated(
    __out IBurnView **ppView
    );
