//-------------------------------------------------------------------------------------------------
// <copyright file="search.h" company="Microsoft">
//    Copyright (c) Microsoft Corporation.  All rights reserved.
// </copyright>
//
// <summary>
//
// </summary>
//-------------------------------------------------------------------------------------------------

#pragma once


#if defined(__cplusplus)
extern "C" {
#endif


// constants

enum BURN_SEARCH_TYPE
{
    BURN_SEARCH_TYPE_NONE,
    BURN_SEARCH_TYPE_DIRECTORY,
    BURN_SEARCH_TYPE_FILE,
    BURN_SEARCH_TYPE_REGISTRY,
};

enum BURN_SEARCH_SCHEDULING
{
    BURN_SEARCH_SCHEDULING_NONE,
    BURN_SEARCH_SCHEDULING_DETECT,
    BURN_SEARCH_SCHEDULING_PREPARE,
};

enum BURN_DIRECTORY_SEARCH_TYPE
{
    BURN_DIRECTORY_SEARCH_TYPE_NONE,
    BURN_DIRECTORY_SEARCH_TYPE_EXISTS,
};

enum BURN_FILE_SEARCH_TYPE
{
    BURN_FILE_SEARCH_TYPE_NONE,
    BURN_FILE_SEARCH_TYPE_EXISTS,
    BURN_FILE_SEARCH_TYPE_VERSION,
};

enum BURN_REGISTRY_SEARCH_TYPE
{
    BURN_REGISTRY_SEARCH_TYPE_NONE,
    BURN_REGISTRY_SEARCH_TYPE_EXISTS,
    BURN_REGISTRY_SEARCH_TYPE_VALUE,
};


// structs

typedef struct _BURN_SEARCH
{
    LPWSTR sczKey;
    LPWSTR sczProperty;
    LPWSTR sczCondition;
    BURN_SEARCH_SCHEDULING Scheduling;

    LPWSTR* rgsczDependencies;
    DWORD cDependencies;

    BURN_SEARCH_TYPE Type;
    union
    {
        struct
        {
            BURN_DIRECTORY_SEARCH_TYPE Type;
            LPWSTR sczPath;
        } DirectorySearch;
        struct
        {
            BURN_FILE_SEARCH_TYPE Type;
            LPWSTR sczPath;
        } FileSearch;
        struct
        {
            BURN_REGISTRY_SEARCH_TYPE Type;
            BURN_VARIANT_TYPE ValueType;
            HKEY hRoot;
            LPWSTR sczKey;
            LPWSTR sczValue;
            BOOL fExpandEnvironment;
        } RegistrySearch;
    };
} BURN_SEARCH;

typedef struct _BURN_SEARCHES
{
    BURN_SEARCH* rgSearches;
    DWORD cSearches;
} BURN_SEARCHES;


// function declarations

HRESULT SearchAdd(
    __in BURN_SEARCHES* pSearches,
    __out BURN_SEARCH** ppSearch
    );
HRESULT SearchAddDependency(
    __in BURN_SEARCH* pSearch,
    __out LPWSTR** ppsczSearchDependency
    );
HRESULT SearchesSort(
    __in BURN_SEARCHES* pSearches
    );
HRESULT SearchesExecute(
    __in BURN_SEARCHES* pSearches,
    __in BURN_PROPERTIES* pProperties,
    __in BURN_SEARCH_SCHEDULING scheduling
    );
void SearchesUninitialize(
    __in BURN_SEARCHES* pSearches
    );
void SearchFree(
    __in BURN_SEARCH* pSearch
    );

#if defined(__cplusplus)
}
#endif
