//-------------------------------------------------------------------------------------------------
// <copyright file="worker.h" company="Microsoft">
//    Copyright (c) Microsoft Corporation.  All rights reserved.
//    
//    The use and distribution terms for this software are covered by the
//    Common Public License 1.0 (http://opensource.org/licenses/cpl.php)
//    which can be found in the file CPL.TXT at the root of this distribution.
//    By using this software in any fashion, you are agreeing to be bound by
//    the terms of this license.
//    
//    You must not remove this notice, or any other, from this software.
// </copyright>
//
// <summary>
// Setup executable worker thread header.
// </summary>
//-------------------------------------------------------------------------------------------------

#pragma once


// debug hook macros

#ifdef DEBUG

extern SETUP_DEBUG_HOOKS EngineDebugHooks;

#define HOOKABLE_CreateProcessW \
    (EngineDebugHooks.pfnCreateProcessW ? EngineDebugHooks.pfnCreateProcessW : CreateProcessW)
#define HOOKABLE_GetExitCodeProcess \
    (EngineDebugHooks.pfnGetExitCodeProcess ? EngineDebugHooks.pfnGetExitCodeProcess : GetExitCodeProcess)
#define HOOKABLE_MsiGetProductInfoW \
    (EngineDebugHooks.pfnMsiGetProductInfoW ? EngineDebugHooks.pfnMsiGetProductInfoW : MsiGetProductInfoW)
#define HOOKABLE_MsiQueryFeatureStateW \
    (EngineDebugHooks.pfnMsiQueryFeatureStateW ? EngineDebugHooks.pfnMsiQueryFeatureStateW : MsiQueryFeatureStateW)
#define HOOKABLE_MsiSetInternalUI \
    (EngineDebugHooks.pfnMsiSetInternalUI ? EngineDebugHooks.pfnMsiSetInternalUI : MsiSetInternalUI)
#define HOOKABLE_MsiSetExternalUIW \
    (EngineDebugHooks.pfnMsiSetExternalUIW ? EngineDebugHooks.pfnMsiSetExternalUIW : MsiSetExternalUIW)
#define HOOKABLE_MsiSetExternalUIRecord \
    (EngineDebugHooks.pfnMsiSetExternalUIRecord ? EngineDebugHooks.pfnMsiSetExternalUIRecord : pEngineInfo->pfnMsiSetExternalUIRecord)
#define HOOKABLE_MsiEnableLogW \
    (EngineDebugHooks.pfnMsiEnableLogW ? EngineDebugHooks.pfnMsiEnableLogW : MsiEnableLogW)
#define HOOKABLE_MsiInstallProductW \
    (EngineDebugHooks.pfnMsiInstallProductW ? EngineDebugHooks.pfnMsiInstallProductW : MsiInstallProductW)
#define HOOKABLE_MsiConfigureProductExW \
    (EngineDebugHooks.pfnMsiConfigureProductExW ? EngineDebugHooks.pfnMsiConfigureProductExW : MsiConfigureProductExW)

#else

#define HOOKABLE_CreateProcessW CreateProcessW
#define HOOKABLE_GetExitCodeProcess GetExitCodeProcess
#define HOOKABLE_MsiGetProductInfoW MsiGetProductInfoW
#define HOOKABLE_MsiQueryFeatureStateW MsiQueryFeatureStateW
#define HOOKABLE_MsiSetInternalUI MsiSetInternalUI
#define HOOKABLE_MsiSetExternalUIW MsiSetExternalUIW
#define HOOKABLE_MsiSetExternalUIRecord (pEngineInfo->pfnMsiSetExternalUIRecord)
#define HOOKABLE_MsiEnableLogW MsiEnableLogW
#define HOOKABLE_MsiInstallProductW MsiInstallProductW
#define HOOKABLE_MsiConfigureProductExW MsiConfigureProductExW

#endif


// structs

typedef struct _SETUP_WORKER_INFORMATION
{
    SETUP_ENGINE_INFORMATION *pEngineInfo;
    HANDLE hInitEvent;
} SETUP_WORKER_INFORMATION;


// function declarations

DWORD WINAPI WorkerThread(
    __in LPVOID pvContext
    );

LRESULT SendSuxMessage(
    __in SETUP_ENGINE_INFORMATION *pEngineInfo,
    __in UINT Msg,
    __in LPVOID pvData
    );


// SUX message wrappers

inline HRESULT SendSuxPreparePackageBeginMessage(
    __in SETUP_ENGINE_INFORMATION *pEngineInfo,
    __in DWORD dwIndex
    )
{
    PREPARE_PACKAGE_BEGIN_STRUCT msgst = { dwIndex };
    return SendSuxMessage(pEngineInfo, SRM_PREPARE_PACKAGE_BEGIN, (LPVOID)&msgst);
}

inline HRESULT SendSuxPreparePackageCompleteMessage(
    __in SETUP_ENGINE_INFORMATION *pEngineInfo,
    __in DWORD dwIndex,
    __in PACKAGE_STATE CurrentState,
    __in ACTION_STATE ActionState,
    __in PACKAGE_STATUS Status
    )
{
    PREPARE_PACKAGE_COMPLETE_STRUCT msgst = { dwIndex, CurrentState, ActionState, Status };
    return SendSuxMessage(pEngineInfo, SRM_PREPARE_PACKAGE_COMPLETE, (LPVOID)&msgst);
}

inline HRESULT SendSuxPrepareFeatureBeginMessage(
    __in SETUP_ENGINE_INFORMATION *pEngineInfo,
    __in DWORD dwPackageIndex,
    __in DWORD dwFeatureIndex
    )
{
    PREPARE_FEATURE_BEGIN_STRUCT msgst = { dwPackageIndex, dwFeatureIndex };
    return SendSuxMessage(pEngineInfo, SRM_PREPARE_FEATURE_BEGIN, (LPVOID)&msgst);
}

inline HRESULT SendSuxPrepareFeatureCompleteMessage(
    __in SETUP_ENGINE_INFORMATION *pEngineInfo,
    __in DWORD dwPackageIndex,
    __in DWORD dwFeatureIndex,
    __in FEATURE_STATE CurrentState,
    __in FEATURE_ACTION ActionState
    )
{
    PREPARE_FEATURE_COMPLETE_STRUCT msgst = { dwPackageIndex, dwFeatureIndex, CurrentState, ActionState };
    return SendSuxMessage(pEngineInfo, SRM_PREPARE_FEATURE_COMPLETE, (LPVOID)&msgst);
}

inline HRESULT SendSuxPrerequisiteCheckMessage(
    __in SETUP_ENGINE_INFORMATION *pEngineInfo,
    __in DWORD dwPackageIndex,
    __in DWORD dwPrerequisitIndex,
    __in BOOL fSucceeded
    )
{
    PREPARE_PREREQUISITE_CHECK_STRUCT msgst = { dwPackageIndex, dwPrerequisitIndex, fSucceeded };
    return SendSuxMessage(pEngineInfo, SRM_PREPARE_PREREQUISITE_CHECK, (LPVOID)&msgst);
}

inline HRESULT SendSuxPrepareDiskCostMessage(
    __in SETUP_ENGINE_INFORMATION *pEngineInfo,
    __in SETUP_DISK_COST_INFO* rgVolumes,
    __in DWORD dwVolumes
    )
{
    PREPARE_DISK_COST_STRUCT msgst = { rgVolumes, dwVolumes };
    return SendSuxMessage(pEngineInfo, SRM_PREPARE_DISK_COST, (LPVOID)&msgst);
}

inline HRESULT SendSuxCachePackageBeginMessage(
    __in SETUP_ENGINE_INFORMATION *pEngineInfo,
    __in DWORD dwIndex
    )
{
    CACHE_PACKAGE_BEGIN_STRUCT msgst = { dwIndex };
    return SendSuxMessage(pEngineInfo, SRM_CACHE_PACKAGE_BEGIN, (LPVOID)&msgst);
}

inline HRESULT SendSuxCachePackageCompleteMessage(
    __in SETUP_ENGINE_INFORMATION *pEngineInfo,
    __in DWORD dwIndex,
    __in PACKAGE_STATUS Status
    )
{
    CACHE_PACKAGE_COMPLETE_STRUCT msgst = { dwIndex, Status };
    return SendSuxMessage(pEngineInfo, SRM_CACHE_PACKAGE_COMPLETE, (LPVOID)&msgst);
}

inline HRESULT SendSuxExecPackageBeginMessage(
    __in SETUP_ENGINE_INFORMATION *pEngineInfo,
    __in DWORD dwIndex,
    __in ACTION_STATE Action,
    __in BOOL fRollback
    )
{
    EXEC_PACKAGE_BEGIN_STRUCT msgst = { dwIndex, Action, fRollback };
    return SendSuxMessage(pEngineInfo, SRM_EXEC_PACKAGE_BEGIN, (LPVOID)&msgst);
}

inline HRESULT SendSuxExecPackageCompleteMessage(
    __in SETUP_ENGINE_INFORMATION *pEngineInfo,
    __in DWORD dwIndex,
    __in DWORD dwCode,
    __in PACKAGE_STATUS Status
    )
{
    EXEC_PACKAGE_COMPLETE_STRUCT msgst = { dwIndex, dwCode, Status };
    return SendSuxMessage(pEngineInfo, SRM_EXEC_PACKAGE_COMPLETE, (LPVOID)&msgst);
}

inline HRESULT SendSuxExecMsiMessage(
    __in SETUP_ENGINE_INFORMATION *pEngineInfo,
    __in UINT uiMsg,
    __in DWORD dwPackageIndex,
    __in UINT uiMessageType,
    __in LPCWSTR wzMessage,
    __in_opt MSIHANDLE hRecord,
    __in LPINT piReturnValue
    )
{
    EXEC_MSI_MESSAGE_STRUCT msgst = { dwPackageIndex, uiMessageType, wzMessage, hRecord, piReturnValue };
    return SendSuxMessage(pEngineInfo, uiMsg, (LPVOID)&msgst);
}

inline HRESULT SendSuxMasterProgressUpdateMessage(
    __in SETUP_ENGINE_INFORMATION *pEngineInfo,
    __in BOOL fReset,
    __in DWORD64 qwTicks,
    __in DWORD64 qwComplete
    )
{
    MASTER_PROGRESS_STRUCT msgst = { fReset, qwTicks, qwComplete };
    //Trace2(REPORT_STANDARD, "PROGRESS: %d, %d", (DWORD)qwTicks, (DWORD)qwComplete);
    return SendSuxMessage(pEngineInfo, SRM_MASTER_PROGRESS_UPDATE, (LPVOID)&msgst);
}

inline HRESULT SendSuxQueryAbort(
    __in SETUP_ENGINE_INFORMATION *pEngineInfo,
    __in LPBOOL pfAbort
    )
{
    QUERY_ABORT_STRUCT msgst = { pfAbort };
    return SendSuxMessage(pEngineInfo, SRM_QUERY_ABORT, (LPVOID)&msgst);
}

inline HRESULT SendSuxSourceResolutionMessage(
    __in SETUP_ENGINE_INFORMATION *pEngineInfo,
    __in LPCWSTR wzFileName,
    __in LPWSTR* ppwzPath,
    __in LPBOOL pfPathProvided
    )
{
    SOURCE_RESOLUTION_STRUCT msgst = { wzFileName, ppwzPath, pfPathProvided};
    return SendSuxMessage(pEngineInfo, SRM_SOURCE_RESOLUTION, (LPVOID)&msgst);
}
