//-------------------------------------------------------------------------------------------------
// <copyright file="setup.h" company="Microsoft">
//    Copyright (c) Microsoft Corporation.  All rights reserved.
//    
//    The use and distribution terms for this software are covered by the
//    Common Public License 1.0 (http://opensource.org/licenses/cpl.php)
//    which can be found in the file CPL.TXT at the root of this distribution.
//    By using this software in any fashion, you are agreeing to be bound by
//    the terms of this license.
//    
//    You must not remove this notice, or any other, from this software.
// </copyright>
//
// <summary>
// Setup chainer/bootstrapper executable header for WiX toolset.
// </summary>
//-------------------------------------------------------------------------------------------------

#pragma once


#define WIXAPI __stdcall


// setup chain constants

enum INSTALL_ACTION
{
    INSTALL_ACTION_INSTALL,
    INSTALL_ACTION_REPAIR,
    INSTALL_ACTION_UPDATE,
    INSTALL_ACTION_UNINSTALL,
    INSTALL_ACTION_MODIFY,
};

enum RESTART_LEVEL
{
    RESTART_REALLY_SUPPRESS,
    RESTART_SUPPRESS,
    RESTART_PROMPT,
    RESTART_FORCE,
};

enum MSI_UI_LEVEL
{
    MSI_UI_LEVEL_DEFAULT,
    MSI_UI_LEVEL_NONE,
    MSI_UI_LEVEL_BASIC,
    MSI_UI_LEVEL_REDUCED,
    MSI_UI_LEVEL_FULL,
};

enum SETUP_PACKAGE_TYPE
{
    SETUP_PACKAGE_TYPE_EXE,
    SETUP_PACKAGE_TYPE_MSI,
    SETUP_PACKAGE_TYPE_MSP,
};

enum FEATURE_STATE
{
    FEATURE_STATE_UNKNOWN,
    FEATURE_STATE_ABSENT,
    FEATURE_STATE_ADVERTISED,
    FEATURE_STATE_LOCAL,
    FEATURE_STATE_SOURCE,
};

enum FEATURE_ACTION
{
    FEATURE_ACTION_NONE,
    FEATURE_ACTION_ADDLOCAL,
    FEATURE_ACTION_ADDSOURCE,
    FEATURE_ACTION_ADDDEFAULT,
    FEATURE_ACTION_REINSTALL,
    FEATURE_ACTION_ADVERTISE,
    FEATURE_ACTION_REMOVE,
};

enum PACKAGE_STATUS
{
    PACKAGE_STATUS_NONE,

    PACKAGE_STATUS_PENDING_PREPARE,
    PACKAGE_STATUS_IGNORED,
    PACKAGE_STATUS_PREREQUISITE_CHECK_FAILED,

    PACKAGE_STATUS_PENDING_CACHE,
    PACKAGE_STATUS_EXTRACTION_FAILED,
    PACKAGE_STATUS_DOWNLOAD_FAILED,

    PACKAGE_STATUS_PENDING_EXECUTE,
    PACKAGE_STATUS_SOURCE_RESOLUTION_FAILED,
    PACKAGE_STATUS_FAILED,
    PACKAGE_STATUS_SUCCEEDED,
    PACKAGE_STATUS_SUCCEEDED_REBOOT_AT_END,
    PACKAGE_STATUS_WARNING,
    PACKAGE_STATUS_ABORTED,

    PACKAGE_STATUS_ROLLBACK_NODOWNGRADE,
    PACKAGE_STATUS_ROLLBACK_FAILED,
    PACKAGE_STATUS_ROLLBACK_SUCCEEDED,
};

enum EXIT_CODE_TYPE
{
    EXIT_CODE_TYPE_NONE,
    EXIT_CODE_TYPE_SUCCESS,
    EXIT_CODE_TYPE_WARNING,
    EXIT_CODE_TYPE_ERROR,
};

enum REGISTRY_SEARCH_TYPE
{
    REGISTRY_SEARCH_TYPE_NONE,
    REGISTRY_SEARCH_TYPE_EXISTS,
    REGISTRY_SEARCH_TYPE_VALUE,
};

enum PACKAGE_SOURCE_TYPE
{
    PACKAGE_SOURCE_TYPE_NONE,
    PACKAGE_SOURCE_TYPE_RELATIVE,
    PACKAGE_SOURCE_TYPE_EMBEDDED,
    PACKAGE_SOURCE_TYPE_DOWNLOAD,
};


// setup worker interaction messages

enum SETUP_WORK_MESSAGES
{
    SWM_UNKNOWN,
    SWM_QUIT,
    SWM_ECHO,
    SWM_PREPARE,
    SWM_CACHE,
    SWM_EXECUTE
};

enum SETUP_RESULT_MESSAGES
{
    SRM_INFORMATION_UPDATED = WM_APP + 1,
    SRM_ECHO,
    SRM_EXEC_FATAL_ERROR,
    SRM_EXEC_ERROR,
    SRM_EXEC_WARNING,
    SRM_EXEC_LOG,
    SRM_EXEC_INFO,
    SRM_EXEC_ACTION_START,
    SRM_EXEC_ACTION_DATA,
    SRM_EXEC_SET_CAPTION,
    SRM_EXEC_SET_CANCEL,
    SRM_EXEC_RESTART_MANAGER_FILE_IN_USE,
    SRM_EXEC_FILE_IN_USE,
    SRM_EXEC_OUT_OF_DISK_SPACE,
    SRM_EXEC_PROGRESS_RESET,
    SRM_EXEC_INITIALIZE,
    SRM_EXEC_TERMINATE,
    SRM_EXEC_SHOW_DIALOG,
    SRM_EXEC_RESOLVE_SOURCE,

    // prepare messages
    SRM_PREPARE_CHAIN_BEGIN,
    SRM_PREPARE_CHAIN_COMPLETE,
    SRM_PREPARE_PACKAGE_BEGIN,
    SRM_PREPARE_PACKAGE_COMPLETE,
    SRM_PREPARE_FEATURE_BEGIN,
    SRM_PREPARE_FEATURE_COMPLETE,
    SRM_PREPARE_PREREQUISITE_CHECK,
    SRM_PREPARE_DISK_COST,

    // cache messages
    SRM_CACHE_CHAIN_BEGIN,
    SRM_CACHE_CHAIN_COMPLETE,
    SRM_CACHE_PACKAGE_BEGIN,
    SRM_CACHE_PACKAGE_COMPLETE,

    // execute messages
    SRM_EXEC_CHAIN_BEGIN,
    SRM_EXEC_CHAIN_COMPLETE,
    SRM_EXEC_PACKAGE_BEGIN,
    SRM_EXEC_PACKAGE_COMPLETE,
    SRM_EXEC_MSI_FATAL_ERROR,
    SRM_EXEC_MSI_ERROR,
    SRM_EXEC_MSI_WARNING,
    SRM_EXEC_MSI_USER,
    SRM_EXEC_MSI_INFO,
    SRM_EXEC_MSI_ACTION_START,
    SRM_EXEC_MSI_ACTION_DATA,

    // general messages
    SRM_MASTER_PROGRESS_UPDATE,
    SRM_QUERY_ABORT,
    SRM_SOURCE_RESOLUTION,
};


// setup chain data structures

typedef struct _SETUP_PROPERTY
{
    LPWSTR wzId;
    VARIANT vtValue;
} SETUP_PROPERTY;

typedef struct _SETUP_REGISTRY_SEARCH
{
    LPWSTR wzId;
    REGISTRY_SEARCH_TYPE Type;
    HKEY hRoot; // HKEY_CURRENT_USER or HKEY_LOCAL_MACHINE
    LPWSTR wzKey;
    LPWSTR wzValue;
    LPWSTR wzProperty;
} SETUP_REGISTRY_SEARCH;

typedef struct _SETUP_DISK_COST
{
    LPWSTR wzPath;
    DWORD64 qwCost;
} SETUP_DISK_COST;

typedef struct _SETUP_PACKAGE_MSI_TRANSFORM
{
    LPCSTR szPath;
    DWORD dwLocaleId;
} SETUP_PACKAGE_MSI_TRANSFORM;

typedef struct _SETUP_PACKAGE_MSI_FEATURE
{
    LPWSTR wzId;
    DWORD64 qwCost;
    BOOL fRepair;

    DWORD cDiskCosts;
    SETUP_DISK_COST* rgDiskCosts;

    FEATURE_STATE RequestedState;
    FEATURE_STATE ExpectedState;
    FEATURE_STATE CurrentState;
    FEATURE_ACTION ExecuteAction;
    FEATURE_ACTION RollbackAction;
} SETUP_PACKAGE_MSI_FEATURE;

typedef struct _SETUP_PACKAGE_MSI_PROPERTY
{
    LPWSTR wzId;
    LPWSTR wzRequestedValue; // used during forward execution
    LPWSTR wzExpectedValue;  // used during rollback
} SETUP_PACKAGE_MSI_PROPERTY;

typedef struct _SETUP_PACKAGE_MSP
{
} SETUP_PACKAGE_MSP;

typedef struct _SETUP_PACKAGE_MSI
{
    GUID guidProductCode;
    DWORD64 qwVersion;
    MSI_UI_LEVEL uiLevel;

    DWORD cTransforms;
    SETUP_PACKAGE_MSI_TRANSFORM* rgTransforms;

    DWORD cProperties;
    SETUP_PACKAGE_MSI_PROPERTY* rgProperties;

    DWORD cFeatures;
    SETUP_PACKAGE_MSI_FEATURE* rgFeatures;

    DWORD dwReturnCode;
} SETUP_PACKAGE_MSI;

typedef struct _SETUP_PACKAGE_EXE_EXIT_CODE
{
    BOOL fWildcard;
    DWORD dwExitCode;
    EXIT_CODE_TYPE Type;
} SETUP_PACKAGE_EXE_EXIT_CODE;

typedef struct _SETUP_PACKAGE_EXE
{
    // TODO: add detection logic to set the version of this package for
    //       later computations.
    DWORD64 qwVersion;
    BOOL fEnableRepair;

    LPWSTR wzInstallCheck;

    LPWSTR wzArguments;
    LPWSTR wzRepairArguments;
    LPWSTR wzUninstallArguments;

    DWORD cExitCodes;
    SETUP_PACKAGE_EXE_EXIT_CODE* rgExitCodes;

    DWORD dwExitCode;
} SETUP_PACKAGE_EXE;

typedef struct _SETUP_PREREQUISITE_CHECK
{
    LPWSTR wzId;
    LPWSTR pwzCondition;
} SETUP_PREREQUISITE_CHECK;

typedef struct _SETUP_PACKAGE
{
    LPWSTR wzId;
    LPWSTR wzSource;
    PACKAGE_SOURCE_TYPE SourceType;
    LPWSTR wzFileName;
    DWORD64 cbFileSize;
    DWORD dwEmbeddedOffset;
    LPWSTR wzExecutePath;
    DWORD dwAttributes; // any combination of the #define SETUP_PACKAGE_ATTRIB
    BOOL fPermanent;
    BOOL fVital;
    BOOL fRepair;
    DWORD64 qwCost;
    DWORD64 qwProgressTicks;
    DWORD64 qwProgressOffset;
    HRESULT hResult;

    SETUP_DISK_COST* rgDiskCosts;
    DWORD cDiskCosts;

    SETUP_PREREQUISITE_CHECK* rgPrereqChecks;
    DWORD cPrereqChecks;

    LPWSTR pwzCondition;

    PACKAGE_STATE RequestedState;
    PACKAGE_STATE ExpectedState;
    PACKAGE_STATE CurrentState;
    ACTION_STATE ActionState;
    ACTION_STATE RollbackAction;

    DWORD64 qwInstalledVersion;

    PACKAGE_STATUS Status;

    SETUP_PACKAGE_TYPE PackageType;
    union
    {
        SETUP_PACKAGE_EXE Exe;
        SETUP_PACKAGE_MSI Msi;
        SETUP_PACKAGE_MSP Msp;
    };
} SETUP_PACKAGE;

typedef struct _SETUP_PACKAGES
{
    //CRITICAL_SECTION cs;

    DWORD cProperties;
    SETUP_PROPERTY* rgProperties;

    DWORD cRegistrySearches;
    SETUP_REGISTRY_SEARCH* rgRegistrySearches;

    DWORD cPackages;
    SETUP_PACKAGE rgPackages[1];
} SETUP_PACKAGES;

typedef struct _SETUP_COMMAND_LINE
{
    HRESULT hrParse;
    HINSTANCE hInstance;
    int nCmdShow;
    INSTALLUILEVEL UILevel;
    INSTALL_ACTION InstallAction;
    RESTART_LEVEL RestartLevel;
    LPWSTR wzLog;
    LPWSTR wzManifest;
    LPWSTR wzProperties;
} SETUP_COMMAND_LINE;

typedef struct _SETUP_DISK_COST_INFO
{
    LPWSTR wzVolumeRootPath;
    DWORD dwVolumeSerialNumber;
    DWORD64 qwCost;
    DWORD64 qwTotal;
    DWORD64 qwFree;
} SETUP_DISK_COST_INFO;


// result message structs

typedef struct _CHAIN_COMPLETE_STRUCT
{
    HRESULT hResult;
} CHAIN_COMPLETE_STRUCT;

typedef struct _PREPARE_PACKAGE_BEGIN_STRUCT
{
    DWORD dwIndex;
} PREPARE_PACKAGE_BEGIN_STRUCT;

typedef struct _PREPARE_PACKAGE_COMPLETE_STRUCT
{
    DWORD dwIndex;
    PACKAGE_STATE CurrentState;
    ACTION_STATE ActionState;
    PACKAGE_STATUS Status;
} PREPARE_PACKAGE_COMPLETE_STRUCT;

typedef struct _PREPARE_FEATURE_BEGIN_STRUCT
{
    DWORD dwPackageIndex;
    DWORD dwFeatureIndex;
} PREPARE_FEATURE_BEGIN_STRUCT;

typedef struct _PREPARE_FEATURE_COMPLETE_STRUCT
{
    DWORD dwPackageIndex;
    DWORD dwFeatureIndex;
    FEATURE_STATE CurrentState;
    FEATURE_ACTION ActionState;
} PREPARE_FEATURE_COMPLETE_STRUCT;

typedef struct _PREPARE_PREREQUISITE_CHECK_STRUCT
{
    DWORD dwPackageIndex;
    DWORD dwPrerequisiteIndex;
    BOOL fSucceeded;
} PREPARE_PREREQUISITE_CHECK_STRUCT;

typedef struct _PREPARE_DISK_COST_STRUCT
{
    SETUP_DISK_COST_INFO* rgVolumes;
    DWORD dwVolumes;
} PREPARE_DISK_COST_STRUCT;

typedef struct _CACHE_PACKAGE_BEGIN_STRUCT
{
    DWORD dwIndex;
} CACHE_PACKAGE_BEGIN_STRUCT;

typedef struct _CACHE_PACKAGE_COMPLETE_STRUCT
{
    DWORD dwIndex;
    PACKAGE_STATUS Status;
} CACHE_PACKAGE_COMPLETE_STRUCT;

typedef struct _EXEC_PACKAGE_BEGIN_STRUCT
{
    DWORD dwIndex;
    ACTION_STATE Action;
    BOOL fRollback;
} EXEC_PACKAGE_BEGIN_STRUCT;

typedef struct _EXEC_PACKAGE_COMPLETE_STRUCT
{
    DWORD dwIndex;
    DWORD dwCode;
    PACKAGE_STATUS Status;
} EXEC_PACKAGE_COMPLETE_STRUCT;

typedef struct _EXEC_MSI_MESSAGE_STRUCT
{
    DWORD dwPackageIndex;
    UINT uiMessageType;
    LPCWSTR wzMessage;
    MSIHANDLE hRecord; // NULL if MSI version < 3.1
    LPINT piReturnValue;
} EXEC_MSI_MESSAGE_STRUCT;

typedef struct _MASTER_PROGRESS_STRUCT
{
    BOOL fReset;
    DWORD64 qwTicks;
    DWORD64 qwComplete;
} MASTER_PROGRESS_STRUCT;

typedef struct _QUERY_ABORT_STRUCT
{
    LPBOOL pfAbort;
} QUERY_ABORT_STRUCT;

typedef struct _SOURCE_RESOLUTION_STRUCT
{
    LPCWSTR wzFileName;
    LPWSTR* ppwzPath;
    LPBOOL pfPathProvided;
} SOURCE_RESOLUTION_STRUCT;

typedef struct _SETUP_COMM_MESSAGE
{
    DWORD dwMessage;
    DWORD cbData;

    BOOL fAllocatedData;
    LPVOID pvData;
} SETUP_COMM_MESSAGE;
