//-------------------------------------------------------------------------------------------------
// <copyright file="sux.cpp" company="Microsoft">
//    Copyright (c) Microsoft Corporation.  All rights reserved.
//    
//    The use and distribution terms for this software are covered by the
//    Common Public License 1.0 (http://opensource.org/licenses/cpl.php)
//    which can be found in the file CPL.TXT at the root of this distribution.
//    By using this software in any fashion, you are agreeing to be bound by
//    the terms of this license.
//    
//    You must not remove this notice, or any other, from this software.
// </copyright>
//
// <summary>
// Setup UX library for bootstrapper in WiX toolset.
// </summary>
//-------------------------------------------------------------------------------------------------

#include "precomp.h"

// prototypes
static HRESULT GetPackageByIndex(
    __in SETUP_PACKAGES *pSetupPackages,
    __in DWORD dwIndex,
    __out SETUP_PACKAGE **ppPackage
    );


HWND SetupUXGetWindow(
    __in HANDLE hSetupUXInformation
    )
{
    SETUP_UX_INFORMATION *pSetupUXInformation = static_cast<SETUP_UX_INFORMATION*>(hSetupUXInformation);
    return pSetupUXInformation->hWnd;
}


DWORD SetupUXGetPackageCount(
    __in HANDLE hSetupPackages
    )
{
    SETUP_PACKAGES *pSetupPackages = static_cast<SETUP_PACKAGES*>(hSetupPackages);
    DWORD cPackages = 0;

    //::EnterCriticalSection(&pSetupPackages->cs);

    cPackages = pSetupPackages->cPackages;

    //::LeaveCriticalSection(&pSetupPackages->cs);
    return cPackages;
}

#if 0
HRESULT SetupUXSetPackageArguments(
    __in HANDLE hSetupPackages,
    __in DWORD dwPackageIndex,
    __in_opt LPCWSTR wzArguments
    )
{
    HRESULT hr = S_OK;
    SETUP_PACKAGES *pSetupPackages = static_cast<SETUP_PACKAGES*>(hSetupPackages);
    SETUP_PACKAGE *pPackage = NULL;

    ::EnterCriticalSection(&pSetupPackages->cs);

    hr = GetPackageByIndex(pSetupPackages, dwPackageIndex, &pPackage);
    ExitOnFailure1(hr, "Failed to get package at index: %d", dwPackageIndex);

    if (wzArguments)
    {
        hr = StrAllocString(&pPackage->wzArguments, wzArguments, 0);
    }
    else
    {
        ReleaseNullStr(pPackage->wzArguments);
    }

LExit:
    ::LeaveCriticalSection(&pSetupPackages->cs);
    return hr;
}
#endif

HRESULT SetupUXSetFeatureActionState(
    __in HANDLE hSetupPackages,
    __in DWORD dwPackageIndex,
    __in LPCWSTR wzFeature,
    __in FEATURE_STATE fsState
    )
{
    HRESULT hr = S_OK;
    SETUP_PACKAGES *pSetupPackages = static_cast<SETUP_PACKAGES*>(hSetupPackages);
    SETUP_PACKAGE *pPackage = NULL;
    SETUP_PACKAGE_MSI_FEATURE *pFeatures = NULL;
    SETUP_PACKAGE_MSI_FEATURE *pFeature = NULL;

    //::EnterCriticalSection(&pSetupPackages->cs);

    hr = GetPackageByIndex(pSetupPackages, dwPackageIndex, &pPackage);
    ExitOnFailure1(hr, "Failed to get package at index: %d", dwPackageIndex);

    if (SETUP_PACKAGE_TYPE_MSI != pPackage->PackageType)
    {
        ExitFunction1(hr = E_FAIL);
    }

    // find existing property
    for (DWORD i = 0; i < pPackage->Msi.cFeatures; i++)
    {
        if (CSTR_EQUAL == ::CompareStringW(LOCALE_INVARIANT, 0, pPackage->Msi.rgFeatures[i].wzId, -1, wzFeature, -1))
        {
            pFeature = &pPackage->Msi.rgFeatures[i];
            break;
        }
    }

    // add property if not found
    if (!pFeature)
    {
        pFeatures = (SETUP_PACKAGE_MSI_FEATURE*)MemAlloc((pPackage->Msi.cFeatures + 1) * sizeof(SETUP_PACKAGE_MSI_FEATURE), TRUE);
        ExitOnNull(pFeatures, hr, E_OUTOFMEMORY, "Failed to allocate new feature array.");

        if (0 != pPackage->Msi.cFeatures)
        {
            ::CopyMemory(pFeatures, pPackage->Msi.rgFeatures, pPackage->Msi.cFeatures * sizeof(SETUP_PACKAGE_MSI_FEATURE));
            MemFree(pPackage->Msi.rgFeatures);
        }

        pFeature = &pFeatures[pPackage->Msi.cFeatures];

        hr = StrAllocString(&pFeature->wzId, wzFeature, 0);
        ExitOnFailure(hr, "Failed to get allocate feature id string.");

        pPackage->Msi.rgFeatures = pFeatures;
        pFeatures = NULL;
        pPackage->Msi.cFeatures++;
    }

    // set property value
    pFeature->RequestedState = fsState;

LExit:
    //::LeaveCriticalSection(&pSetupPackages->cs);
    ReleaseMem(pFeatures);
    return hr;
}

#if 0
HRESULT SetupUXSetPackageProperty(
    __in HANDLE hSetupPackages,
    __in DWORD dwPackageIndex,
    __in LPCWSTR wzProperty,
    __in_opt LPCWSTR wzValue
    )
{
    HRESULT hr = S_OK;
    SETUP_PACKAGES *pSetupPackages = static_cast<SETUP_PACKAGES*>(hSetupPackages);
    SETUP_PACKAGE *pPackage = NULL;
    SETUP_PACKAGE_MSI_PROPERTY *pProperties = NULL;
    SETUP_PACKAGE_MSI_PROPERTY *pProperty = NULL;

    //::EnterCriticalSection(&pSetupPackages->cs);

    hr = GetPackageByIndex(pSetupPackages, dwPackageIndex, &pPackage);
    ExitOnFailure1(hr, "Failed to get package at index: %d", dwPackageIndex);

    if (SETUP_PACKAGE_TYPE_MSI != pPackage->PackageType)
    {
        ExitFunction1(hr = E_FAIL);
    }

    // find existing property
    for (DWORD i = 0; i < pPackage->Msi.cProperties; i++)
    {
        if (CSTR_EQUAL == ::CompareStringW(LOCALE_INVARIANT, 0, pPackage->Msi.rgProperties[i].wzId, -1, wzProperty, -1))
        {
            pProperty = &pPackage->Msi.rgProperties[i];
            break;
        }
    }

    // add property if not found
    if (!pProperty)
    {
        pProperties = (SETUP_PACKAGE_MSI_PROPERTY*)MemAlloc((pPackage->Msi.cProperties + 1) * sizeof(SETUP_PACKAGE_MSI_PROPERTY), TRUE);
        ExitOnNull(pProperties, hr, E_OUTOFMEMORY, "Failed to allocate new property array.");

        if (0 != pPackage->Msi.cProperties)
        {
            ::CopyMemory(pProperties, pPackage->Msi.rgProperties, pPackage->Msi.cProperties * sizeof(SETUP_PACKAGE_MSI_PROPERTY));
            MemFree(pPackage->Msi.rgProperties);
        }

        pProperty = &pProperties[pPackage->Msi.cProperties];

        hr = StrAllocString(&pProperty->wzId, wzProperty, 0);
        ExitOnFailure(hr, "Failed to get allocate property id string.");

        pPackage->Msi.rgProperties = pProperties;
        pProperties = NULL;
        pPackage->Msi.cProperties++;
    }

    // set property value
    if (wzValue)
    {
        hr = StrAllocString(&pProperty->wzValue, wzValue, 0);
    }
    else
    {
        ReleaseNullStr(pProperty->wzValue);
    }

LExit:
    //::LeaveCriticalSection(&pSetupPackages->cs);
    ReleaseMem(pProperties);
    return hr;
}
#endif


HRESULT SetupUXSetPackageActionState(
    __in HANDLE hSetupPackages,
    __in DWORD dwPackageIndex,
    __in PACKAGE_STATE action
    )
{
    HRESULT hr = S_OK;
    SETUP_PACKAGES *pSetupPackages = static_cast<SETUP_PACKAGES*>(hSetupPackages);
    SETUP_PACKAGE *pPackage = NULL;

    //::EnterCriticalSection(&pSetupPackages->cs);

    hr = GetPackageByIndex(pSetupPackages, dwPackageIndex, &pPackage);
    ExitOnFailure1(hr, "Failed to get package at index: %d", dwPackageIndex);

    //if (ACTION_STATE_UNINSTALL == action && PACKAGE_STATE_ABSENT == pPackage->CurrentState)
    //{
    //    hr = E_INVALIDARG;
    //}
    //else
    {
        pPackage->RequestedState = action;
    }

LExit:
    //::LeaveCriticalSection(&pSetupPackages->cs);
    return hr;
}


HRESULT SetupUXPostMessage(
    __in HANDLE hSetupUXInformation,
    __in DWORD dwMessage,
    __in_opt LPVOID pvData,
    __in DWORD cbData
    )
{
    HRESULT hr = S_OK;
    SETUP_ENGINE_INFORMATION *pSetupUXInformation = static_cast<SETUP_ENGINE_INFORMATION*>(hSetupUXInformation);
    SETUP_COMM_MESSAGE msg = { 0 };
    DWORD cbWritten = 0;

    msg.dwMessage = dwMessage;
    msg.cbData = cbData;
    msg.pvData = pvData;

    if (!::WriteFile(pSetupUXInformation->hPipe, &msg, sizeof(DWORD) * 2, &cbWritten, NULL))
    {
        ExitWithLastError(hr, "Failed to write message.");
    }

    if (!::WriteFile(pSetupUXInformation->hPipe, msg.pvData, cbData, &cbWritten, NULL))
    {
        ExitWithLastError(hr, "Failed to write message data.");
    }

LExit:
    return hr;
}


// NOTE: This function assumes that it is already in the 
// critical section.
static HRESULT GetPackageByIndex(
    __in SETUP_PACKAGES *pSetupPackages,
    __in DWORD dwIndex,
    __out SETUP_PACKAGE **ppPackage
    )
{
    HRESULT hr = S_OK;

    if (dwIndex < pSetupPackages->cPackages)
    {
        *ppPackage = pSetupPackages->rgPackages + dwIndex;
    }
    else
    {
        hr = E_INVALIDARG;
    }

    return hr;
}
