//-------------------------------------------------------------------------------------------------
// <copyright file="BurnControllerTestShim.cpp" company="Microsoft">
//    Copyright (c) Microsoft Corporation.  All rights reserved.
//    
//    The use and distribution terms for this software are covered by the
//    Common Public License 1.0 (http://opensource.org/licenses/cpl.php)
//    which can be found in the file CPL.TXT at the root of this distribution.
//    By using this software in any fashion, you are agreeing to be bound by
//    the terms of this license.
//    
//    You must not remove this notice, or any other, from this software.
// </copyright>
//-------------------------------------------------------------------------------------------------

#include "precomp.h"

namespace MS
{
namespace Tools
{
namespace WindowsInstallerXml
{
namespace Test
{
namespace Bootstrapper
{
//
// Prepare - call IBurnController::Prepare().
//
void BurnControllerTestShim::Prepare(int packageIndex, RequestState requested)
{
    interior_ptr<IBurnController*> ppController = &(this->m_pController);
    IBurnController* pThisController = *ppController;

    HRESULT hr = S_OK;
    DWORD dwPackageIndex = packageIndex;

    hr = pThisController->Prepare(dwPackageIndex, static_cast<REQUEST_STATE>(requested));
    if (FAILED(hr))
    {
        throw gcnew BurnTestShimException(hr, "Failed while calling prepare controller.");
    }
}


//
// Cache - call IBurnController::Cache().
//
void BurnControllerTestShim::Cache(int packageIndex)
{
    interior_ptr<IBurnController*> ppController = &(this->m_pController);
    IBurnController* pThisController = *ppController;

    HRESULT hr = S_OK;
    DWORD dwPackageIndex = packageIndex;

    hr = pThisController->Cache(dwPackageIndex);
    if (FAILED(hr))
    {
        throw gcnew BurnTestShimException(hr, "Failed while calling cache controller.");
    }
}


//
// Elevate - call IBurnController::Elevate().
//
void BurnControllerTestShim::Elevate()
{
    interior_ptr<IBurnController*> ppController = &(this->m_pController);
    IBurnController* pThisController = *ppController;

    HRESULT hr = S_OK;
    hr = pThisController->Elevate(NULL);
    if (FAILED(hr))
    {
        throw gcnew BurnTestShimException(hr, "Failed while calling elevate controller.");
    }
}


//
// Execute - call IBurnController::Execute().
//
void BurnControllerTestShim::Execute(RequestState requested)
{
    interior_ptr<IBurnController*> ppController = &(this->m_pController);
    IBurnController* pThisController = *ppController;

    HRESULT hr = S_OK;

    hr = pThisController->Execute(static_cast<REQUEST_STATE>(requested));
    if (FAILED(hr))
    {
        throw gcnew BurnTestShimException(hr, "Failed while calling execute controller.");
    }
}


//
// Run - call IBurnController::Run().
//
void BurnControllerTestShim::Run()
{
    interior_ptr<IBurnController*> ppController = &(this->m_pController);
    IBurnController* pThisController = *ppController;

    HRESULT hr = S_OK;

    hr = pThisController->Run();
    if (FAILED(hr))
    {
        throw gcnew BurnTestShimException(hr, "Failed while running controller.");
    }
}


//
// OnCachePackages - call IBurnControllerTestable::OnCachePackages().
//
void BurnControllerTestShim::OnCachePackages(int packageIndex)
{
    interior_ptr<IBurnController*> ppController = &(this->m_pController);
    IBurnController* pThisController = *ppController;

    HRESULT hr;
    IBurnControllerTestable* pTestableController = NULL;
    try
    {
        hr = pThisController->QueryInterface(IID_PPV_ARGS(&pTestableController));
        if (FAILED(hr))
        {
            throw gcnew BurnTestShimException(hr, "Failed to query for testable controller.");
        }

        DWORD dwPackageIndex = packageIndex;
        pTestableController->OnCachePackages(dwPackageIndex);
    }
    finally
    {
        ReleaseObject(pTestableController);
    }
}


//
// Constructors
//
BurnControllerTestShim::BurnControllerTestShim(BurnModelTestShim^ model, IBurnViewShim^ viewShim)
{
    this->Construct(model, viewShim, nullptr);
}


BurnControllerTestShim::BurnControllerTestShim(BurnModelTestShim^ model, IBurnViewShim^ viewShim, IInstallationAbstractionLayerShim^ ialShim)
{
    this->Construct(model, viewShim, ialShim);
}


BurnControllerTestShim::BurnControllerTestShim(IBurnController* pController)
{
    pController->AddRef();
    this->m_pController = pController;
}


//
// Destructor
//
BurnControllerTestShim::~BurnControllerTestShim()
{
    this->Destroy();
}


//
// Finalizer
//
BurnControllerTestShim::!BurnControllerTestShim()
{
    this->Destroy();
}


//
// Construct - common code for constructors.
//
void BurnControllerTestShim::Construct(BurnModelTestShim^ model, IBurnViewShim^ viewShim, IInstallationAbstractionLayerShim^ ialShim)
{
    interior_ptr<IBurnModel*> ppModel = &(model->m_pModel);
    IBurnView* pView = viewShim->GetView();
    interior_ptr<IBurnView*> ppView = &(pView);

    HRESULT hr = S_OK;
    IInstallationAbstractionLayer* pInstallationAbstractLayer = NULL;
    IBurnExecuteEngine* pExeEngine = NULL;
    IBurnExecuteEngine* pMsiEngine = NULL;
    IBurnCacheEngine* pExtractEngine = NULL;
    IBurnCacheEngine* pDownloadEngine = NULL;

    try
    {
        if (nullptr == ialShim)
        {
            hr = BurnCreateInstallationAbstractionLayer(&pInstallationAbstractLayer);
            if (FAILED(hr))
            {
                throw gcnew BurnTestShimException(hr, "Failed to create installation abstract layer.");
            }
        }
        else
        {
            pInstallationAbstractLayer = ialShim->GetInstallationAbstractionLayer();
        }

        hr = BurnCreateExeEngine(pInstallationAbstractLayer, &pExeEngine);
        if (FAILED(hr))
        {
            throw gcnew BurnTestShimException(hr, "Failed to create exe engine.");
        }

        hr = BurnCreateMsiEngine(pInstallationAbstractLayer, &pMsiEngine);
        if (FAILED(hr))
        {
            throw gcnew BurnTestShimException(hr, "Failed to create msi engine.");
        }

        hr = BurnCreateExtractEngine(&pExtractEngine);
        if (FAILED(hr))
        {
            throw gcnew BurnTestShimException(hr, "Failed to create extract engine.");
        }

        hr = BurnCreateDownloadEngine(&pDownloadEngine);
        if (FAILED(hr))
        {
            throw gcnew BurnTestShimException(hr, "Failed to create download engine.");
        }

        IBurnController* x = NULL;
        hr = BurnCreateController(*ppModel, *ppView, pExeEngine, pMsiEngine, pExtractEngine, pDownloadEngine, pInstallationAbstractLayer, &x);
        if (FAILED(hr))
        {
            throw gcnew BurnTestShimException(hr, "Failed to create controller.");
        }

        this->m_pController = x;
    }
    finally
    {
        ReleaseObject(pDownloadEngine);
        ReleaseObject(pExtractEngine);
        ReleaseObject(pInstallationAbstractLayer);
        ReleaseObject(pMsiEngine);
        ReleaseObject(pExeEngine);
        ReleaseObject(pView);
    }
}


//
// Destroy - cleans up the object (called by destructor and finalizer).
//
void BurnControllerTestShim::Destroy()
{
    ReleaseNullObject(this->m_pController);
}
}
}
}
}
}
