//-------------------------------------------------------------------------------------------------
// <copyright file="BurnPackageTestShim.cpp" company="Microsoft">
//    Copyright (c) Microsoft Corporation.  All rights reserved.
//    
//    The use and distribution terms for this software are covered by the
//    Common Public License 1.0 (http://opensource.org/licenses/cpl.php)
//    which can be found in the file CPL.TXT at the root of this distribution.
//    By using this software in any fashion, you are agreeing to be bound by
//    the terms of this license.
//    
//    You must not remove this notice, or any other, from this software.
// </copyright>
//-------------------------------------------------------------------------------------------------

#include "precomp.h"

namespace MS
{
namespace Tools
{
namespace WindowsInstallerXml
{
namespace Test
{
namespace Bootstrapper
{
//
// InstalledVersion - call IBurnPackage::GetInstalledState().
//
Version^ BurnPackageTestShim::InstalledVersion::get()
{
    interior_ptr<IBurnPackage*> ppPackage = &(this->m_pPackage);
    IBurnPackage* pThisPackage = *ppPackage;

    DWORD64 qwVersion = 0;
    pThisPackage->GetInstalledState(NULL, &qwVersion);

    return gcnew Version(qwVersion >> 48 & 0xFFFF, qwVersion >> 32 & 0xFFFF, qwVersion >> 16 & 0xFFFF, qwVersion & 0xFFFF);
}


//
// PackageVersion - call IBurnPackage::GetVersion().
//
Version^ BurnPackageTestShim::PackageVersion::get()
{
    interior_ptr<IBurnPackage*> ppPackage = &(this->m_pPackage);
    IBurnPackage* pThisPackage = *ppPackage;

    DWORD64 qwVersion = 0;
    pThisPackage->GetVersion(&qwVersion);

    return gcnew Version(qwVersion >> 48 & 0xFFFF, qwVersion >> 32 & 0xFFFF, qwVersion >> 16 & 0xFFFF, qwVersion & 0xFFFF);
}


//
// Properties - call IBurnPackage::GetProperties().
//
BurnPropertiesTestShim^ BurnPackageTestShim::Properties::get()
{
    interior_ptr<IBurnPackage*> ppPackage = &(this->m_pPackage);
    IBurnPackage* pThisPackage = *ppPackage;

    IBurnProperties* pProperties = NULL;

    try
    {
        HRESULT hr = pThisPackage->GetProperties(&pProperties);
        if (FAILED(hr))
        {
            throw gcnew BurnTestShimException(hr, "Failed to get package properties.");
        }

        return gcnew BurnPropertiesTestShim(pProperties);
    }
    finally
    {
        ReleaseObject(pProperties);
    }
}


//
// GetPayload - call IBurnPackage::GetPayload().
//
BurnPayloadTestShim^ BurnPackageTestShim::GetPayload(int index)
{
    interior_ptr<IBurnPackage*> ppPackage = &(this->m_pPackage);
    IBurnPackage* pThisPackage = *ppPackage;

    IBurnPayload* pPayload = NULL;

    try
    {
        HRESULT hr = pThisPackage->GetPayload(index, &pPayload);
        if (FAILED(hr))
        {
            throw gcnew BurnTestShimException(hr, "Failed to get payload.");
        }

        return gcnew BurnPayloadTestShim(pPayload);
    }
    finally
    {
        ReleaseObject(pPayload);
    }
}


String^ BurnPackageTestShim::LogPath::get()
{
    interior_ptr<IBurnPackage*> ppPackage = &(this->m_pPackage);
    IBurnPackage* pThisPackage = *ppPackage;

    LPWSTR scz = NULL;
    BURN_LOGGING_LEVEL logLevel = BURN_LOGGING_LEVEL_UNKNOWN;

    try
    {
        HRESULT hr = pThisPackage->GetLogPath(&scz, &logLevel);
        if (FAILED(hr))
        {
            throw gcnew BurnTestShimException(hr, "Failed to get package log path.");
        }

        return gcnew String(scz);
    }
    finally
    {
        ReleaseStr(scz);
    }
}

LoggingLevel BurnPackageTestShim::LogLevel::get()
{
    interior_ptr<IBurnPackage*> ppPackage = &(this->m_pPackage);
    IBurnPackage* pThisPackage = *ppPackage;

    LPWSTR scz = NULL;
    BURN_LOGGING_LEVEL logLevel = BURN_LOGGING_LEVEL_UNKNOWN;

    try
    {
        HRESULT hr = pThisPackage->GetLogPath(&scz, &logLevel);
        if (FAILED(hr))
        {
            throw gcnew BurnTestShimException(hr, "Failed to get package log path.");
        }

        return (LoggingLevel)logLevel;
    }
    finally
    {
        ReleaseStr(scz);
    }
}

bool BurnPackageTestShim::IsPerMachine::get()
{
    interior_ptr<IBurnPackage*> ppPackage = &(this->m_pPackage);
    IBurnPackage* pThisPackage = *ppPackage;

    return pThisPackage->IsPerMachine() ? true : false;
}

void BurnPackageTestShim::AddPayload(BurnPayloadTestShim^ payload)
{
    interior_ptr<IBurnPackage*> ppPackage = &(this->m_pPackage);
    IBurnPackage* pThisPackage = *ppPackage;
    interior_ptr<IBurnPayload*> ppPayload = &(payload->m_pPayload);

    HRESULT hr;
    IBurnPackageTestable* pTestablePackage = NULL;
    try
    {
        hr = pThisPackage->QueryInterface(IID_PPV_ARGS(&pTestablePackage));
        if (FAILED(hr))
        {
            throw gcnew BurnTestShimException(hr, "Failed to add payload to package.");
        }

        hr = pTestablePackage->AddPayloadForTestingPurposes(*ppPayload);
        if (FAILED(hr))
        {
            throw gcnew BurnTestShimException(hr, "Failed to add payload to package.");
        }
    }
    finally
    {
        ReleaseObject(pTestablePackage);
    }
}

//
// Create - creates the appropriate package type for the IBurnPackage.
//
BurnPackageTestShim^ BurnPackageTestShim::Create(IBurnPackage* pPackage)
{
    IBurnExePackage* pExePackage = NULL;
    IBurnMsiPackage* pMsiPackage = NULL;

    try
    {
        HRESULT hr = pPackage->QueryInterface(__uuidof(IBurnExePackage), reinterpret_cast<void**>(&pExePackage));
        if (FAILED(hr))
        {
            hr = pPackage->QueryInterface(__uuidof(IBurnMsiPackage), reinterpret_cast<void**>(&pMsiPackage));
            if (FAILED(hr))
            {
                throw gcnew BurnTestShimException(hr, "Failed to determine type of package (exe or msi).");
            }

            return gcnew BurnMsiPackageTestShim(pMsiPackage);
        }
        else
        {
            return gcnew BurnExePackageTestShim(pExePackage);
        }
    }
    finally
    {
        ReleaseObject(pMsiPackage);
        ReleaseObject(pExePackage);
    }
}


//
// Constructor
//
BurnPackageTestShim::BurnPackageTestShim()
{
    this->m_pPackage = NULL;
}


//
// Destructor
//
BurnPackageTestShim::~BurnPackageTestShim()
{
    this->Destroy();
}


//
// Finalizer
//
BurnPackageTestShim::!BurnPackageTestShim()
{
    this->Destroy();
}


//
// Destroy - cleans up the object (called by destructor and finalizer).
//
void BurnPackageTestShim::Destroy()
{
    ReleaseNullObject(this->m_pPackage);
}
}
}
}
}
}
