//-------------------------------------------------------------------------------------------------
// <copyright file="BurnPayloadTestShim.cpp" company="Microsoft">
//    Copyright (c) Microsoft Corporation.  All rights reserved.
//    
//    The use and distribution terms for this software are covered by the
//    Common Public License 1.0 (http://opensource.org/licenses/cpl.php)
//    which can be found in the file CPL.TXT at the root of this distribution.
//    By using this software in any fashion, you are agreeing to be bound by
//    the terms of this license.
//    
//    You must not remove this notice, or any other, from this software.
// </copyright>
//-------------------------------------------------------------------------------------------------

#include "precomp.h"

namespace MS
{
namespace Tools
{
namespace WindowsInstallerXml
{
namespace Test
{
namespace Bootstrapper
{
//
// CompletedPath - call IBurnPayload::GetCompletedPath().
//
String^ BurnPayloadTestShim::CompletedPath::get()
{
    interior_ptr<IBurnPayload*> ppPayload = &(this->m_pPayload);
    IBurnPayload* pThisPayload = *ppPayload;

    LPWSTR scz = NULL;
    try
    {
        HRESULT hr = pThisPayload->GetCompletedPath(&scz);
        if (FAILED(hr))
        {
            throw gcnew BurnTestShimException(hr, "Failed to get payload completed path.");
        }

        return gcnew String(scz);
    }
    finally
    {
        ReleaseStr(scz);
    }
}


//
// CompletedRoot - call IBurnPayload::SetCompletedRoot().
//
void BurnPayloadTestShim::CompletedRoot::set(String^ value)
{
    interior_ptr<IBurnPayload*> ppPayload = &(this->m_pPayload);
    IBurnPayload* pThisPayload = *ppPayload;
    pin_ptr<const WCHAR> wz = PtrToStringChars(value);

    HRESULT hr = pThisPayload->SetCompletedRoot(wz);
    if (FAILED(hr))
    {
        throw gcnew BurnTestShimException(hr, "Failed to set payload completed root.");
    }
}


//
// SourcePath - call IBurnPayload::GetSource().
//
String^ BurnPayloadTestShim::SourcePath::get()
{
    interior_ptr<IBurnPayload*> ppPayload = &(this->m_pPayload);
    IBurnPayload* pThisPayload = *ppPayload;

    LPWSTR scz = NULL;
    try
    {
        HRESULT hr = pThisPayload->GetSource(NULL, &scz);
        if (FAILED(hr))
        {
            throw gcnew BurnTestShimException(hr, "Failed to get payload source path.");
        }

        return gcnew String(scz);
    }
    finally
    {
        ReleaseStr(scz);
    }
}


//
// SourceType - call IBurnPayload::GetSource().
//
PayloadSourceType BurnPayloadTestShim::SourceType::get()
{
    interior_ptr<IBurnPayload*> ppPayload = &(this->m_pPayload);
    IBurnPayload* pThisPayload = *ppPayload;

    SOURCE_TYPE type;
    HRESULT hr = pThisPayload->GetSource(&type, NULL);
    if (FAILED(hr))
    {
        throw gcnew BurnTestShimException(hr, "Failed to get payload source path.");
    }

    return (PayloadSourceType)type;
}


//
// WorkingPath - call IBurnPayload::GetWorkingPath().
//
String^ BurnPayloadTestShim::WorkingPath::get()
{
    interior_ptr<IBurnPayload*> ppPayload = &(this->m_pPayload);
    IBurnPayload* pThisPayload = *ppPayload;

    LPWSTR scz = NULL;
    try
    {
        HRESULT hr = pThisPayload->GetWorkingPath(&scz);
        if (FAILED(hr))
        {
            throw gcnew BurnTestShimException(hr, "Failed to get payload working path.");
        }

        return gcnew String(scz);
    }
    finally
    {
        ReleaseStr(scz);
    }
}

//
// Create - creates payload for the IBurnPayload.
//
//BurnPayloadTestShim^ BurnPayloadTestShim::Create(IBurnPayload* pPayload)
//{
//    return gcnew BurnPayloadTestShim(pPayload);
//}


//
// Constructor
//
BurnPayloadTestShim::BurnPayloadTestShim(PayloadSourceType sourceType, Guid guid, String^ sourcePath, String^ localPath, UInt64 size, array<Byte>^ digest, bool temporary)
{
    SOURCE_TYPE type = static_cast<SOURCE_TYPE>(sourceType);
    array<Byte>^ guidData = guid.ToByteArray();
    pin_ptr<Byte> data = &(guidData[0]);

    pin_ptr<const WCHAR> wzSourcePath = PtrToStringChars(sourcePath);
    pin_ptr<const WCHAR> wzLocalPath = PtrToStringChars(localPath);
    DWORD64 cbSize = size;
    pin_ptr<Byte> rgDigest = &(digest[0]);
    DWORD cbDigest = digest->Length;

    IBurnPayload* x = NULL;
    HRESULT hr = BurnCreatePayload(type, *reinterpret_cast<GUID*>(data), wzSourcePath, wzLocalPath, cbSize, reinterpret_cast<LPCBYTE>(rgDigest), cbDigest, temporary ? TRUE : FALSE, &x);
    if (FAILED(hr))
    {
        throw gcnew BurnTestShimException(hr, "Failed to create payload.");
    }

    this->m_pPayload = x;
}

BurnPayloadTestShim::BurnPayloadTestShim(IBurnPayload* pPayload)
{
    pPayload->AddRef();
    this->m_pPayload = pPayload;
}


//
// Destructor
//
BurnPayloadTestShim::~BurnPayloadTestShim()
{
    this->Destroy();
}


//
// Finalizer
//
BurnPayloadTestShim::!BurnPayloadTestShim()
{
    this->Destroy();
}


//
// Destroy - cleans up the object (called by destructor and finalizer).
//
void BurnPayloadTestShim::Destroy()
{
    ReleaseNullObject(this->m_pPayload);
}
}
}
}
}
}
