//-------------------------------------------------------------------------------------------------
// <copyright file="BurnPropertiesTestShim.cpp" company="Microsoft">
//    Copyright (c) Microsoft Corporation.  All rights reserved.
//    
//    The use and distribution terms for this software are covered by the
//    Common Public License 1.0 (http://opensource.org/licenses/cpl.php)
//    which can be found in the file CPL.TXT at the root of this distribution.
//    By using this software in any fashion, you are agreeing to be bound by
//    the terms of this license.
//    
//    You must not remove this notice, or any other, from this software.
// </copyright>
//-------------------------------------------------------------------------------------------------

#include "precomp.h"

namespace MS
{
namespace Tools
{
namespace WindowsInstallerXml
{
namespace Test
{
namespace Bootstrapper
{
__int64 BurnPropertiesTestShim::GetNumericValueByName(String^ name)
{
    pin_ptr<const WCHAR> wzName = PtrToStringChars(name);
    interior_ptr<IBurnProperties*> ppProperties = &(this->m_pProperties);
    IBurnProperties* pThisProperties = *ppProperties;

    LONGLONG llValue = 0;
    HRESULT hr = pThisProperties->GetNumericValueByName(wzName, &llValue);
    if (FAILED(hr))
    {
        throw gcnew BurnTestShimException(hr, "Failed to get numeric property value by name.");
    }

    return (__int64)llValue;
}


String^ BurnPropertiesTestShim::GetStringValueByName(String^ name)
{
    pin_ptr<const WCHAR> wzName = PtrToStringChars(name);
    interior_ptr<IBurnProperties*> ppProperties = &(this->m_pProperties);
    IBurnProperties* pThisProperties = *ppProperties;

    LPWSTR scz = NULL;
    try
    {
        HRESULT hr = pThisProperties->GetStringValueByName(wzName, &scz);
        if (FAILED(hr))
        {
            throw gcnew BurnTestShimException(hr, "Failed to get string property value by name.");
        }

        return gcnew String(scz);
    }
    finally
    {
        ReleaseStr(scz);
    }
}


unsigned __int64 BurnPropertiesTestShim::GetVersionValueByName(String^ name)
{
    pin_ptr<const WCHAR> wzName = PtrToStringChars(name);
    interior_ptr<IBurnProperties*> ppProperties = &(this->m_pProperties);
    IBurnProperties* pThisProperties = *ppProperties;

    DWORD64 qwValue = 0;
    HRESULT hr = pThisProperties->GetVersionValueByName(wzName, &qwValue);
    if (FAILED(hr))
    {
        throw gcnew BurnTestShimException(hr, "Failed to get numeric property value by name.");
    }

    return (unsigned __int64)qwValue;
}


String^ BurnPropertiesTestShim::GetFormattedValueByName(String^ name)
{
    pin_ptr<const WCHAR> wzName = PtrToStringChars(name);
    interior_ptr<IBurnProperties*> ppProperties = &(this->m_pProperties);
    IBurnProperties* pThisProperties = *ppProperties;

    LPWSTR scz = NULL;
    try
    {
        HRESULT hr = pThisProperties->GetFormattedValueByName(wzName, &scz);
        if (FAILED(hr))
        {
            throw gcnew BurnTestShimException(hr, "Failed to get formatted property value by name.");
        }

        return gcnew String(scz);
    }
    finally
    {
        ReleaseStr(scz);
    }
}


void BurnPropertiesTestShim::SetNumeric(String^ name, __int64 value)
{
    pin_ptr<const WCHAR> wzName = PtrToStringChars(name);
    LONGLONG llValue = (LONGLONG)value;
    interior_ptr<IBurnProperties*> ppProperties = &(this->m_pProperties);
    IBurnProperties* pThisProperties = *ppProperties;

    HRESULT hr = pThisProperties->SetNumeric(wzName, llValue);
        if (FAILED(hr))
        {
            throw gcnew BurnTestShimException(hr, "Failed to set numeric property value.");
        }
}


void BurnPropertiesTestShim::SetString(String^ name, String^ value)
{
    pin_ptr<const WCHAR> wzName = PtrToStringChars(name);
    pin_ptr<const WCHAR> wzValue = PtrToStringChars(value);
    interior_ptr<IBurnProperties*> ppProperties = &(this->m_pProperties);
    IBurnProperties* pThisProperties = *ppProperties;

    HRESULT hr = pThisProperties->SetString(wzName, wzValue);
        if (FAILED(hr))
        {
            throw gcnew BurnTestShimException(hr, "Failed to set numeric property value.");
        }
}


void BurnPropertiesTestShim::SetVersion(String^ name, unsigned __int64 value)
{
    pin_ptr<const WCHAR> wzName = PtrToStringChars(name);
    DWORD64 qwValue = (DWORD64)value;
    interior_ptr<IBurnProperties*> ppProperties = &(this->m_pProperties);
    IBurnProperties* pThisProperties = *ppProperties;

    HRESULT hr = pThisProperties->SetVersion(wzName, qwValue);
        if (FAILED(hr))
        {
            throw gcnew BurnTestShimException(hr, "Failed to set numeric property value.");
        }
}


String^ BurnPropertiesTestShim::FormatString(String^ in)
{
    pin_ptr<const WCHAR> wzIn = PtrToStringChars(in);
    interior_ptr<IBurnProperties*> ppProperties = &(this->m_pProperties);
    IBurnProperties* pThisProperties = *ppProperties;

    LPWSTR scz = NULL;
    try
    {
        HRESULT hr = pThisProperties->FormatString(wzIn, &scz);
        if (FAILED(hr))
        {
            throw gcnew BurnTestShimException(hr, "Failed to get format string.");
        }

        return gcnew String(scz);
    }
    finally
    {
        ReleaseStr(scz);
    }
}


String^ BurnPropertiesTestShim::EscapeString(String^ in)
{
    pin_ptr<const WCHAR> wzIn = PtrToStringChars(in);
    interior_ptr<IBurnProperties*> ppProperties = &(this->m_pProperties);
    IBurnProperties* pThisProperties = *ppProperties;

    LPWSTR scz = NULL;
    try
    {
        HRESULT hr = pThisProperties->EscapeString(wzIn, &scz);
        if (FAILED(hr))
        {
            throw gcnew BurnTestShimException(hr, "Failed to get escape string.");
        }

        return gcnew String(scz);
    }
    finally
    {
        ReleaseStr(scz);
    }
}


bool BurnPropertiesTestShim::EvaluateCondition(String^ expr)
{
    pin_ptr<const WCHAR> wzExpr = PtrToStringChars(expr);
    interior_ptr<IBurnProperties*> ppProperties = &(this->m_pProperties);
    IBurnProperties* pThisProperties = *ppProperties;

    BOOL f = FALSE;
    HRESULT hr = pThisProperties->EvaluateCondition(wzExpr, &f);
    if (FAILED(hr))
    {
        throw gcnew BurnTestShimException(hr, "Failed to evaluate condition.");
    }

    return f ? true : false;
}


//
// Constructor
//
BurnPropertiesTestShim::BurnPropertiesTestShim()
{
    IBurnProperties* x = NULL;
    HRESULT hr = BurnCreateProperties(&x);
    if (FAILED(hr))
    {
        throw gcnew BurnTestShimException(hr, "Failed to create properties.");
    }

    this->m_pProperties = x;
}


//
// Constructor
//
BurnPropertiesTestShim::BurnPropertiesTestShim(IBurnProperties* pProperties)
{
    pProperties->AddRef();
    this->m_pProperties = pProperties;
}


//
// Destructor
//
BurnPropertiesTestShim::~BurnPropertiesTestShim()
{
    this->Destroy();
}


//
// Finalizer
//
BurnPropertiesTestShim::!BurnPropertiesTestShim()
{
    this->Destroy();
}


//
// Destroy - cleans up the object (called by destructor and finalizer).
//
void BurnPropertiesTestShim::Destroy()
{
    ReleaseNullObject(this->m_pProperties);
}
}
}
}
}
}
